"""
# 作 者：84028
# 时 间：2024/2/28 21:09
# tsdpsdk
"""
from datetime import datetime
from typing import Any

import tgw
from tushare.subs.model.tick import TsTick, TsTickIdx


def get_ts_code(_type: str, _code: str)->str:
    if _type == tgw.MarketType.kNEEQ:
        # 北交所
        return f'{_code}.BJ'
    elif _type == tgw.MarketType.kSSE:
        # 上交所
        return f'{_code}.SH'
    elif _type == tgw.MarketType.kSZSE:
        # 深交所
        return f'{_code}.SZ'
    else:
        raise Exception("未知类型，请联系管理员添加对应数据的解析！")


def get_tgw_type_and_code(_code: str) -> (Any, str):
    _code, _ext = _code.split('.', 1)
    _ext = _ext.upper()
    if _ext == 'BJ':
        # 北交所
        return tgw.MarketType.kNEEQ, _code
    elif _ext == 'SH':
        # 上交所
        return tgw.MarketType.kSSE, _code
    elif _ext == 'SZ':
        # 深交所
        return tgw.MarketType.kSZSE, _code
    else:
        raise Exception(f"未知类型，请联系管理员添加对应数据的解析！--{_ext}")


def convert_tick_index(tgw_data: dict) -> TsTickIdx:
    """
    index_demo = {
        'market_type': 101,
        'security_code': '000300',
        'orig_time': 20240227133145220, # 交易所行情数据时间
        'trading_phase_code': '',
        'pre_close_index': 3453358500,
        'open_index': 3440051600,
        'high_index': 3483385500,
        'low_index': 3437442200,
        'last_index': 3474362400,
        'close_index': 0,
        'total_volume_trade': 8934304600,
        'total_value_trade': 15930131603140000,
        'variety_category': 5
    }
    """
    item = TsTickIdx(
        ts_code=get_ts_code(tgw_data.get("market_type"), tgw_data.get("security_code")),
        name=None,
        trade_time=datetime.strptime(str(tgw_data.get('orig_time'))[:-3], '%Y%m%d%H%M%S'),
        last_price=tgw_data.get('last_index')/1000000,
        pre_close_price=tgw_data.get('pre_close_index')/1000000,
        open_price=tgw_data.get('open_index')/1000000,
        high_price=tgw_data.get('high_index')/1000000,
        low_price=tgw_data.get('low_index')/1000000,
        close_price=tgw_data.get('close_index')/1000000,
        volume=tgw_data.get('total_volume_trade')/100,
        amount=tgw_data.get('total_value_trade')/100000,
    )
    return item


def convert_tick_stock(tgw_data: dict) -> TsTick:
    """
    tick_demo = {
        'market_type': 101,
        'security_code': '600000',
        'variety_category': 1,
        'orig_time': 20240227133145832,             # 交易所行情数据时间
        'trading_phase_code': 'T111',
        'pre_close_price': 7100000,
        'open_price': 7070000,
        'high_price': 7150000,
        'low_price': 7060000,
        'last_price': 7110000,
        'close_price': 0,
        'total_volume_trade': 1621371900,
        'total_value_trade': 11528767100000,
        'bid_price1': 7110000,
        'bid_price2': 7100000,
        'bid_price3': 7090000,
        'bid_price4': 7080000,
        'bid_price5': 7070000,
        'bid_price6': 0,
        'bid_price7': 0,
        'bid_price8': 0,
        'bid_price9': 0,
        'bid_price10': 0,
        'bid_volume1': 20000,
        'bid_volume2': 26810000,
        'bid_volume3': 38450000,
        'bid_volume4': 33100000,
        'bid_volume5': 35720000,
        'bid_volume6': 0,
        'bid_volume7': 0,
        'bid_volume8': 0,
        'bid_volume9': 0,
        'bid_volume10': 0,
        'offer_price1': 7120000,
        'offer_price2': 7130000,
        'offer_price3': 7140000,
        'offer_price4': 7150000,
        'offer_price5': 7160000,
        'offer_price6': 0,
        'offer_price7': 0,
        'offer_price8': 0,
        'offer_price9': 0,
        'offer_price10': 0,
        'offer_volume1': 25400000,
        'offer_volume2': 28180000,
        'offer_volume3': 24750000,
        'offer_volume4': 63481800,
        'offer_volume5': 15723900,
        'offer_volume6': 0,
        'offer_volume7': 0,
        'offer_volume8': 0,
        'offer_volume9': 0,
        'offer_volume10': 0,
        'num_trades': 12339,
        'IOPV': 0,
        'high_limited': 7810000,
        'low_limited': 6390000
    }
    """
    extra = {}
    for i in range(1, 11):
        extra[f'ask_price{i}'] = tgw_data.get(f'offer_price{i}')/1000000
        extra[f'ask_volume{i}'] = tgw_data.get(f'offer_volume{i}')/100
        extra[f'bid_price{i}'] = tgw_data.get(f'bid_price{i}')/1000000
        extra[f'bid_volume{i}'] = tgw_data.get(f'bid_volume{i}')/100
    item = TsTick(
        ts_code=get_ts_code(tgw_data.get("market_type"), tgw_data.get("security_code")),
        name=None,
        trade_time=datetime.strptime(str(tgw_data.get('orig_time'))[:-3], '%Y%m%d%H%M%S'),
        pre_close_price=tgw_data.get("pre_close_price")/1000000,
        last_price=tgw_data.get("last_price")/1000000,
        open_price=tgw_data.get("open_price")/1000000,
        high_price=tgw_data.get("high_price")/1000000,
        low_price=tgw_data.get("low_price")/1000000,
        close_price=tgw_data.get("close_price")/1000000,
        volume=tgw_data.get("total_volume_trade")/100,
        amount=tgw_data.get("total_value_trade")/1000000,
        count=tgw_data.get("num_trades"),
        **extra
    )
    return item



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
token 验证
Created on 2023/12/06
@author: Monday
@group : waditu
@contact:
"""
from typing import Optional
from tushare.util.upass import get_token, set_token
import requests
from requests.adapters import HTTPAdapter, Retry
from tushare.stock import cons as ct


# 自定义异常类，用于权限不足时抛出异常
class PermissionError(Exception):
    pass


# 装饰器函数，用于进行权限验证
def require_permission(event_name, event_detail):
    def decorator(func):
        def wrapper(*args, **kwargs):
            # event_name = kwargs["event_name"]
            # event_detail = kwargs["event_detail"]
            # 检查用户权限
            token = get_token()
            if token:
                try:
                    r = verify_token(token, event_name, event_detail).json()
                except Exception as err:
                    raise PermissionError(f"验证token出错，{err}")
                if r.get("message") == "success":
                    # 如果有足够权限，调用原始函数
                    return func(*args, **kwargs)
                else:
                    print(f"验证token出错，{r.text}")
                    raise PermissionError(f"{r['msg']}")
            else:
                raise PermissionError(ct.TOKEN_ERR_MSG)

        wrapper.__doc__ = func.__doc__
        return wrapper

    return decorator


def verify_token(token: Optional[str] = None, event_name: Optional[str] = "string",
                 event_detail: Optional[str] = "string"):
    session = requests.Session()
    retries = Retry(total=3, backoff_factor=1)
    session.mount('http://', HTTPAdapter(max_retries=retries))
    session.mount('https://', HTTPAdapter(max_retries=retries))
    headers = {
        "accept": "application/json",
        "Content-Type": "application/json"
    }
    url = "https://api.tushare.pro/dataapi/sdk-event"
    data = {"user_token": token,
            "event_name": event_name,
            "event_detail": event_detail}
    response = session.post(url, headers=headers, json=data)
    return response


# 使用装饰器进行权限验证
@require_permission(event_name="xxx", event_detail="xxx")
def admin_only_function():
    print("ssss")


if __name__ == '__main__':
    set_token("asdasdasa")
    admin_only_function()  # 正常情况，输出执行信息



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Created on 2023/12/06
@author: Monday
@group : waditu
Desc: 腾讯-股票-实时行情-成交明细
成交明细-每个交易日 16:00 提供当日数据
港股报价延时 15 分钟
"""
import warnings
import pandas as pd
import requests
from io import StringIO
from tushare.util.verify_token import require_permission
from tushare.util.format_stock_code import format_stock_code
from tushare.stock.rtq_vars import zh_sina_a_stock_cookies, zh_sina_a_stock_headers
import time
from typing import Optional
from tushare.util.form_date import get_current_date
from tushare.stock import rtq_vars
from tushare.util.format_stock_code import symbol_verify

headers = {
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36 Edg/119.0.0.0"
}


@require_permission(event_name="realtime_tick", event_detail="个股历史分笔数据")
def realtime_tick(ts_code: str = "000001.SZ", src: Optional[str] = "tx",
                  page_count: Optional[int] = None) -> pd.DataFrame:
    """
    历史分笔数据
    :param ts_code: 股票代码
    :type ts_code: str
    :param src: 来源  腾讯财经tx   新浪财经sina
    :type src: str
    :param page_count: 限制页数
    :type page_count: str
    :return: 历史分笔数据
    :rtype: pandas.DataFrame
        1、TIME : 成交时间
        2、PRICE : 成交价格
        3、PCHANGE : 涨跌幅
        4、CHANGE : 价格变动
        5、VOLUME : 成交量(手)
        6、AMOUNT : 成交额(元)
        7、TYPE : 性质
    """
    symbol = symbol_verify(ts_code)
    if src == "sina":
        return get_stock_sina_a_divide_amount(symbol, page_count)
    else:
        return get_stock_tx_a_divide_amount(symbol, page_count)


def get_stock_tx_a_divide_amount(symbol: str = "sz000001", page_count: Optional[int] = None) -> pd.DataFrame:
    """
    腾讯财经-历史分笔数据
    https://gu.qq.com/sz300494/gp/detail
    :param symbol: 股票代码
    :type symbol: str
    :param page_count: 限制页数
    :type page_count: str
    :return: 历史分笔数据
    :rtype: pandas.DataFrame
    """
    symbols = str(symbol).lower().split(".")
    symbol = f"{symbols[1]}{symbols[0]}"
    big_df = pd.DataFrame()
    page = 0
    warnings.warn("正在下载数据，请稍等")
    while True:
        try:
            url = "http://stock.gtimg.cn/data/index.php"
            params = {
                "appn": "detail",
                "action": "data",
                "c": symbol,
                "p": page,
            }
            r = requests.get(url, headers=headers, params=params)
            text_data = r.text
            temp_df = (
                pd.DataFrame(eval(text_data[text_data.find("["):])[1].split("|"))
                    .iloc[:, 0]
                    .str.split("/", expand=True)
            )
            page += 1
            big_df = pd.concat([big_df, temp_df], ignore_index=True)
            time.sleep(0.5)
            if page_count and page >= page_count:
                break
        except:
            break
    if not big_df.empty:
        big_df = big_df.iloc[:, 1:].copy()
        # big_df.columns = ["成交时间", "成交价格", "价格变动", "成交量", "成交金额", "性质"]
        big_df.columns = rtq_vars.TICK_COLUMNS

        big_df.reset_index(drop=True, inplace=True)
        property_map = {
            "S": "卖盘",
            "B": "买盘",
            "M": "中性盘",
        }
        # big_df["性质"] = big_df["性质"].map(property_map)
        big_df["TYPE"] = big_df["TYPE"].map(property_map)
        big_df = big_df.astype(
            {
                "TIME": str,
                "PRICE": float,
                "CHANGE": float,
                "VOLUME": int,
                "AMOUNT": int,
                "TYPE": str,
            }
        )
    return big_df


def get_stock_sina_a_divide_amount(symbol: str = "sz000001", page_count: Optional[int] = None, ) -> pd.DataFrame:
    """
    腾新浪财经-历史分笔数据
    https://vip.stock.finance.sina.com.cn/quotes_service/view/vMS_tradedetail.php?symbol=sh688553
    :param symbol: 股票代码
    :type symbol: str
    :param page_count: 限制页数
    :type page_count: str
    :return: 历史分笔数据
    :rtype: pandas.DataFrame
    """
    warnings.warn("正在下载数据，请稍等")
    symbols = str(symbol).lower().split(".")
    symbol = f"{symbols[1]}{symbols[0]}"
    page = 0
    big_df = pd.DataFrame()
    while True:
        try:
            url = "https://vip.stock.finance.sina.com.cn/quotes_service/view/vMS_tradedetail.php"
            params = {
                "symbol": symbol,
                "date": get_current_date(date_format="%Y-%m-%d"),
                "page": page
            }
            response = requests.get(url, headers=zh_sina_a_stock_headers, cookies=zh_sina_a_stock_cookies,
                                    params=params)
            temp_df = (pd.read_html(StringIO(response.content.decode("GBK")))[3])
            big_df = pd.concat([big_df, temp_df], ignore_index=True)
            page += 1
            if page_count and page >= page_count:
                break
        except:
            break
        time.sleep(0.5)
    if not big_df.empty:
        big_df = big_df.iloc[:, 0:].copy()
        # big_df.columns = ["成交时间", "成交价格", "涨跌幅", "价格变动", "成交量(手)", "成交额(元)", "性质"]
        big_df.columns = rtq_vars.TODAY_TICK_COLUMNS
        big_df.reset_index(drop=True, inplace=True)
        # big_df = big_df.astype(
        #     {
        #         "成交时间": str,
        #         "成交价格": float,
        #         "涨跌幅": str,
        #         "价格变动": str,
        #         "成交量(手)": int,
        #         "成交额(元)": int,
        #         "性质": str,
        #     }
        # )
        big_df = big_df.astype(
            {
                "TIME": str,
                "PRICE": float,
                "PCHANGE": str,
                "CHANGE": str,
                "VOLUME": int,
                "AMOUNT": int,
                "TYPE": str,
            }
        )
    return big_df


if __name__ == '__main__':
    df = realtime_tick(ts_code="000001.SZ", src="tx", page_count=1)
    print(help(realtime_tick))
    print(df)



# -*- coding: utf-8 -*-
"""
Created on 2021-12-13 16:29:25
---------
@summary: 股票代码 code 格式化
---------
@author: yangyx01
"""
import re


def format_stock_code(x, special=""):
    """
    股票代码 code 格式化
    @param x:
    @param special:
    @return:
    """
    if special:
        x = str(x)
        if "行业" in special or "概念" in special:
            x = x.zfill(5) if len(x) < 5 else x
            return '%s.TI' % x
        elif "港股" in special:
            r = re.search(r"(\d+)", str(x), re.S | re.M)
            if not r:
                return x
            else:
                x = r.group(1)
                x = x.zfill(5) if len(x) < 5 else x
            return '%s.HK' % x
        elif special == "场外基金" or special == "热基":
            x = x.zfill(5) if len(x) < 5 else x
            return '%s.OF' % x
        elif special == "可转债":
            if str(x).startswith('11'):
                x = x.zfill(5) if len(x) < 5 else x
                return '%s.SH' % x
            elif str(x).startswith('12'):
                x = x.zfill(5) if len(x) < 5 else x
                return '%s.SZ' % x

    if not x[0].isdigit():
        r = re.search(r"(\d+)", str(x), re.S | re.M)
        if not r:
            return x.upper()
        else:
            x = r.group(1)  # # return f"{r.group(1)}.{str(x).split(r.group(1))[0]}"

    if not x[0].isdigit():
        return x.upper()
    if str(x[0]) in ['5', '6', '9']:
        x = x.zfill(5) if len(x) < 5 else x
        return '%s.SH' % x
    elif str(x[0]) in ['8', '4']:
        x = x.zfill(5) if len(x) < 5 else x
        return '%s.BJ' % x
    else:
        x = x.zfill(5) if len(x) < 5 else x
        return '%s.SZ' % x


def a_symbol_verify(x):
    l = str(x).upper().split(".")
    if not x[-1].isdigit():
        return True
    elif len(l) == 2:
        return True
    else:
        return False


def symbol_verify(x):
    ss = x.split(",")
    if len(ss) > 1:
        if a_symbol_verify(ss[0]):
            return x
        else:
            raise '请按照 "000001.SZ,000001.SZ,000001.SZ" 格式传入symbol'
    else:
        if a_symbol_verify(x):
            return x
        else:
            raise '请按照 "000001.SZ" 格式传入symbol'


def verify_stock_or_index(x):
    """
    判断代码是否是 股票 True  指数 False
    @param x:
    @return:
    """
    x = str(x).upper()
    if x.startswith('39') and x.endswith('SZ'):
        return True
    elif x.startswith('30') and x.endswith('SZ'):
        return True
    elif x.startswith('0') and x.endswith('SH'):
        return False
    elif x.startswith('0') and x.endswith('SZ'):
        return True
    elif x.endswith("SH"):
        return False
    # elif x.startswith('6') and x.endswith('SH'):
    #     return True
    elif x.startswith('8') and x.endswith('BJ'):
        return True
    elif x.startswith('4') and x.endswith('BJ'):
        return True
    elif x.startswith('9') and x.endswith('CSI'):
        return False
    else:
        return True


if __name__ == '__main__':
    # s = symbol_verify("13.SH")
    # # s = symbol_verify("sh13")
    # s = symbol_verify("sz000001")
    # s = symbol_verify("000001.SZ")
    # s = symbols_f("000001", special="港股")
    s = verify_stock_or_index(x="399005.SZ")
    print(s)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Created on 2023/12/06
@author: Monday
@group : waditu
@contact:
Desc: 实时行情配置文件
"""
TICK_COLUMNS = ['TIME', 'PRICE', 'CHANGE', 'VOLUME', 'AMOUNT', 'TYPE']
TODAY_TICK_COLUMNS = ['TIME', 'PRICE', 'PCHANGE', 'CHANGE', 'VOLUME', 'AMOUNT', 'TYPE']

# zh-sina-a
zh_sina_a_stock_url = "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeData"
zh_sina_a_stock_payload = {
    "page": "1",
    "num": "80",
    "sort": "symbol",
    "asc": "1",
    "node": "hs_a",
    "symbol": "",
    "_s_r_a": "page"
}
zh_sina_a_stock_count_url = "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeStockCount?node=hs_a"
zh_sina_a_stock_headers = {
    "authority": "vip.stock.finance.sina.com.cn",
    "accept": "*/*",
    "accept-language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6",
    "content-type": "application/x-www-form-urlencoded",
    # "referer": "https://vip.stock.finance.sina.com.cn/mkt/",
    # "sec-ch-ua": "\"Microsoft Edge\";v=\"119\", \"Chromium\";v=\"119\", \"Not?A_Brand\";v=\"24\"",
    # "sec-ch-ua-mobile": "?0",
    # "sec-ch-ua-platform": "\"Windows\"",
    # "sec-fetch-dest": "empty",
    # "sec-fetch-mode": "cors",
    # "sec-fetch-site": "same-origin",
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36 Edg/119.0.0.0"
}
zh_sina_a_stock_cookies = {
    "SINAGLOBAL": "119.253.37.47_1644226852.508455",
    "__bid_n": "18450c0cd85e35c7a94207",
    "SGUID": "1667802124734_57135459",
    "UOR": ",,",
    "U_TRS1": "000000b2.9f5130ad.644b86b5.59248297",
    "FPTOKEN": "uUvE76pfAI7Jn5FFqInwSdRtqTyLuL6pdbfc+x7roC/Cpq02sGKHOVBuDqX/cKmBhQoZsBDCVIcATeySQAv+5tBq6PrNHaCYc4QsF52US/Wr3r8Cm59WbQrDmdDS3PqZpDuWkrt1rqVmi4llaINOSWOnBHROpQon1cDv67tj7ExYL8WNCE/lwJRwHmNWMVwYhNbTV4YIcPF7rNuvdqoBpEmosmMgOcH5xtUi6odo51pnKjBC5YkARH+vWoDLsR4NTSu6WSR5KbZ04ia/4UQWZ8I/EWvyqhqtCnIlrWUiSbsuPPdIKtb2WSuIYW2HwR8Phqeasf3vqLukS+0yHphVhJkUFWI1auczbXZQvVs8Yyaxnxytn1BeuY0QSMaQtZ7WpHpDw/OciAwlxsnTC3D6qg==|5cED1wgbzTZjJ/dqIVeem7K6xn6k7piH0T+g7xbaU7I=|10|5a3963e2a78e2a5d10021960ecf84860",
    "SUB": "_2AkMT-zNof8NxqwFRmP0SzWLjaYRzwgnEieKlp8KzJRMyHRl-yD9kqkEytRB6OHsdh6VxEajRUyiJp4p33F_Ga6PI0ilQ",
    "Hm_lvt_b82ffdf7cbc70caaacee097b04128ac1": "1688714348",
    "ULV": "1699583417865:11:2:2::1698999895443",
    "Hm_lvt_fcf72dc8287d20a78b3dfd301a50cbf8": "1698373541,1699583419",
    "Apache": "220.231.47.169_1701671587.570418"
}

dc_headers = {
    "Connection": "keep-alive",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36",
    "Accept": "*/*",
    "Sec-Fetch-Site": "same-site",
    "Sec-Fetch-Mode": "no-cors",
    "Sec-Fetch-Dest": "script",
    "Referer": "https://quote.eastmoney.com/kcb/688720.html",
    "Accept-Language": "zh-CN,zh;q=0.9"
}
dc_cookies = {
    "qgqp_b_id": "cf8b058a05d005ca7fb2afc14957f250",
    "st_si": "72907886672492",
    "st_asi": "delete",
    "HAList": "ty-1-688720-N%u827E%u68EE%2Cty-0-873122-%u4E2D%u7EBA%u6807",
    "st_pvi": "02194384728897",
    "st_sp": "2023-12-06%2016%3A05%3A53",
    "st_inirUrl": "https%3A%2F%2Fquote.eastmoney.com%2Fcenter%2Fgridlist.html",
    "st_sn": "11",
    "st_psi": "20231206170058129-113200313000-8845421016"
}

LIVE_DATA_COLS = ['NAME', 'OPEN', 'PRE_CLOSE', 'PRICE', 'HIGH', 'LOW', 'BID', 'ASK', 'VOLUME', 'AMOUNT', 'B1_V', 'B1_P',
                  'B2_V', 'B2_P', 'B3_V', 'B3_P', 'B4_V', 'B4_P', 'B5_V', 'B5_P', 'A1_V', 'A1_P', 'A2_V', 'A2_P',
                  'A3_V', 'A3_P', 'A4_V', 'A4_P', 'A5_V', 'A5_P', 'DATE', 'TIME', 'TS_CODE']

LIVE_DATA_COLS_REINDEX = ['NAME', 'TS_CODE', 'DATE', 'TIME', 'OPEN', 'PRE_CLOSE', 'PRICE', 'HIGH', 'LOW', 'BID', 'ASK',
                          'VOLUME', 'AMOUNT', 'B1_V', 'B1_P',
                          'B2_V', 'B2_P', 'B3_V', 'B3_P', 'B4_V', 'B4_P', 'B5_V', 'B5_P', 'A1_V', 'A1_P', 'A2_V',
                          'A2_P',
                          'A3_V', 'A3_P', 'A4_V', 'A4_P', 'A5_V', 'A5_P']



"""
# 作 者：84028
# 时 间：2024/2/21 22:16
# tsdpsdk
"""
# -*- coding: utf-8 -*-

# ------------------------------
# @Time    : 2023/6/15
# @Author  : gao
# @File    : tgw_login.py
# @Project : AmazingQuant
# ------------------------------
import tgw


class TgwLogSpi(tgw.ILogSpi):
    def __init__(self, log_level=2):
        self.log_level = log_level
        super().__init__()

    def OnLog(self, level, log, len):
        pass

    def OnLogon(self, data):
        pass


def login(username, password, host, port):
    log_spi = TgwLogSpi()
    tgw.SetLogSpi(log_spi)

    # 第二步，登录
    cfg = tgw.Cfg()
    cfg.username = username
    cfg.password = password

    cfg.server_vip = host
    cfg.server_port = port
    success = tgw.Login(cfg, tgw.ApiMode.kInternetMode)

    if not success:
        print("login fail")
        exit(0)



"""
# 作 者：84028
# 时 间：2024/2/28 20:51
# tsdpsdk
"""
import time
from concurrent.futures import ThreadPoolExecutor
from typing import Union

import tgw
from ..model.tick import TsTick, TsTickIdx
from .convert import convert_tick_index, convert_tick_stock, get_tgw_type_and_code
from .login import login
from ... import get_token, set_token
from ...util.verify_token import verify_token

threadpool = ThreadPoolExecutor(5)


class TgwSubscribe(tgw.IPushSpi):
    def __init__(self, username, password, host, port=8600) -> None:
        super().__init__()
        self.callback_map = {}
        self.username = username
        self.password = password
        self.host = host
        self.port = port

    def process_data(self, data):
        _type = data.get('variety_category')
        item = None
        if _type == tgw.VarietyCategory.kIndex:
            item = convert_tick_index(data)
        elif _type == tgw.VarietyCategory.kStock:
            item = convert_tick_stock(data)
        if item:
            func = self.callback_map.get(item.ts_code)
            if func and callable(func):
                func(item)

    def OnMDSnapshot(self, data, err):
        if data:
            threadpool.submit(self.process_data, data[0])
            pass
        else:
            print(err)

    def OnMDIndexSnapshot(self, data, err):
        if data:
            threadpool.submit(self.process_data, data[0])
        else:
            print(err)

    def register(self, ts_codes):
        def decorator(func):
            for ts_code in ts_codes:
                self.callback_map[ts_code] = func
            return func
        return decorator

    def run(self):
        token = get_token()
        r = verify_token(token, 'tgw_subscribe', self.username)
        if r.status_code == 200:
            resp_data = r.json()
            if resp_data.get("message") != "success":
                print(f"验证token出错，{r.text}")
                raise PermissionError(f"请先设置Tushare的token，{r['msg']}")

        login(username=self.username, password=self.password, host=self.host, port=self.port)
        items = []
        for ts_code in self.callback_map:
            t, c = get_tgw_type_and_code(ts_code)
            item = tgw.SubscribeItem()
            item.flag = tgw.SubscribeDataType.kSnapshot
            item.category_type = tgw.VarietyCategory.kStock
            item.security_code = c
            item.market = t
            items.append(item)
            item = tgw.SubscribeItem()
            item.flag = tgw.SubscribeDataType.kIndexSnapshot
            item.category_type = tgw.VarietyCategory.kIndex
            item.security_code = c
            item.market = t
            items.append(item)

        self.SetDfFormat(False)
        success = tgw.Subscribe(items, self)
        if success != tgw.ErrorCode.kSuccess:
            print(tgw.GetErrorMsg(success))
        while True:
            time.sleep(10)


def demo():
    set_token('xxxxxx')
    sub = TgwSubscribe(username='username', password='password', host='host', port=8600)

    @sub.register(ts_codes=['600000.SH', '000300.SH'])
    def just_print_index(data: Union[TsTick, TsTickIdx]):
        print(data)

    sub.run()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
所有 A 股的实时行情数据
Created on 2023/12/06
@author: Monday
@group : waditu
@contact:
"""
import time
import pandas as pd
import requests
from tqdm import tqdm
from typing import Optional
import re
from tushare.util.format_stock_code import symbol_verify
from tushare.util.format_stock_code import format_stock_code, verify_stock_or_index
from tushare.util.form_date import timestemp_to_time
from tushare.stock import rtq_vars as rtqv
from tushare.util.verify_token import require_permission
from tushare.stock import cons as ct
from tushare.stock.rtq_vars import (
    zh_sina_a_stock_payload,
    zh_sina_a_stock_url,
    zh_sina_a_stock_count_url,
    zh_sina_a_stock_headers,
    zh_sina_a_stock_cookies,

)


def _random(n=13):
    from random import randint
    start = 10 ** (n - 1)
    end = (10 ** n) - 1
    return str(randint(start, end))


def _get_current_timestamp():
    return str(int(time.time() * 1000))


@require_permission(event_name="realtime_quote", event_detail="个股实时交易数据")
def realtime_quote(ts_code="688553.SH", src="sina", ):
    """
        获取实时交易数据 getting real time quotes data
       用于跟踪交易情况（本次执行的结果-上一次执行的数据）
    Parameters
    ------
        ts_code : string
        src : sina ，dc

    return
    -------
        DataFrame 实时交易数据
              属性:0：name，股票名字
            1：open，今日开盘价
            2：pre_close，昨日收盘价
            3：price，当前价格
            4：high，今日最高价
            5：low，今日最低价
            6：bid，竞买价，即“买一”报价
            7：ask，竞卖价，即“卖一”报价
            8：volumn，成交量 maybe you need do volumn/100
            9：amount，成交金额（元 CNY）
            10：b1_v，委买一（笔数 bid volume）
            11：b1_p，委买一（价格 bid price）
            12：b2_v，“买二”
            13：b2_p，“买二”
            14：b3_v，“买三”
            15：b3_p，“买三”
            16：b4_v，“买四”
            17：b4_p，“买四”
            18：b5_v，“买五”
            19：b5_p，“买五”
            20：a1_v，委卖一（笔数 ask volume）
            21：a1_p，委卖一（价格 ask price）
            ...
            30：date，日期；
            31：time，时间；
        """
    symbols = symbol_verify(ts_code)
    if src == "sina":
        return get_realtime_quotes_sina(symbols)
    else:
        return get_realtime_quotes_dc(symbols)


sina_stock_code = {}


def get_realtime_quotes_sina(symbol="688553"):  # "688553"
    global sina_stock_code
    symbols = []
    syms = []
    for i in [j for j in symbol.split(",")]:
        s = i.split(".")
        sina_stock_code[s[0]] = s[1].upper()
        symbols.append(s[1].lower() + s[0])
        syms.append(s[0])
    # symbols = re.search(r"(\d+)", str(symbols), re.S | re.M).group(1)
    symbols_list = ''
    if isinstance(symbols, list) or isinstance(symbols, set) or \
            isinstance(symbols, tuple) or isinstance(symbols, pd.Series):
        for code in symbols:
            symbols_list += code + ','
    else:
        symbols_list = symbols
    symbols_list = symbols_list[:-1] if len(symbols_list) > 8 else symbols_list
    root_url = ct.LIVE_DATA_URL % (ct.P_TYPE['http'], ct.DOMAINS['sinahq'], _get_current_timestamp(), symbols_list)
    response = requests.get(root_url,
                            headers={
                                "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36 Edg/119.0.0.0",
                                'host': 'hq.sinajs.cn',
                                'referer': 'https://finance.sina.com.cn/'
                            })
    text = response.content.decode('GBK')
    reg = re.compile(r'\="(.*?)\";')
    data = reg.findall(text)
    data_list = []
    syms_list = []
    for index, row in enumerate(data):
        if len(row) > 1:
            data_list.append([astr for astr in row.split(',')[:33]])
            syms_list.append(syms[index])
    if len(syms_list) == 0:
        return None
    df = pd.DataFrame(data_list, columns=ct.LIVE_DATA_COLS)
    df = df.drop('s', axis=1)
    df['code'] = syms_list
    ls = [cls for cls in df.columns if '_v' in cls]
    for txt in ls:
        df[txt] = df[txt].map(lambda x: x[:-2])
    df.columns = rtqv.LIVE_DATA_COLS
    df["TS_CODE"] = df["TS_CODE"].apply(format_sina_stock_code)
    df["DATE"] = df["DATE"].apply(format_date_str)
    new_order = rtqv.LIVE_DATA_COLS_REINDEX
    df = df[new_order]
    # 指定要转换为 float 类型的列
    cols_to_convert = ['OPEN', 'PRE_CLOSE', 'PRICE', 'HIGH', 'LOW', 'BID', 'ASK',
                       'VOLUME', 'AMOUNT', 'B1_V', 'B1_P',
                       'B2_V', 'B2_P', 'B3_V', 'B3_P', 'B4_V', 'B4_P', 'B5_V', 'B5_P', 'A1_V', 'A1_P', 'A2_V',
                       'A2_P',
                       'A3_V', 'A3_P', 'A4_V', 'A4_P', 'A5_V', 'A5_P']
    # 使用 to_numeric() 方法将指定的列转换为 float 类型，并将非数值类型的数据转换为 NaN
    df[cols_to_convert] = df[cols_to_convert].apply(pd.to_numeric, errors='coerce')
    # 使用 fillna() 方法将 NaN 值替换为 0
    df_filled = df.fillna(0)
    return df_filled


def format_sina_stock_code(x):
    ts_code = f"{x}.{sina_stock_code[x]}"
    return ts_code


def format_date_str(date_str):
    return date_str.replace("-", "")


@require_permission(event_name="realtime_list", event_detail="A股所有实时交易数据")
def realtime_list(src: Optional[str] = None, interval: Optional[int] = 3,
                  page_count: Optional[int] = None, proxies: Optional[dict] = {}) -> pd.DataFrame:
    """
    沪深京 A 股 all -实时行情
    @param src:  数据源 新浪sina |东方财富 dc
    @param interval:  分页采集时间间隔（默认3秒翻译一夜）
    @param page_count:  限制抓取的页数（仅对新浪有效）
    @param proxies:  设置代理 防止被封禁
    @return: 按涨跌幅 倒序排序
    -------
        DataFrame 实时交易数据
        东方财富：
            2、代码:TS_CODE
            3、名称:NAME
            4、最新价:PRICE
            5、涨跌幅:PCT_CHANGE
            6、涨跌额:CHANGE
            7、成交量:VOLUME
            8、成交额:AMOUNT
            9、振幅:SWING
            10、最高:HIGH
            11、最低:LOW
            12、今开:OPEN
            13、昨收:CLOSE
            14、量比:VOL_RATIO
            15、换手率:TURNOVER_RATE
            16、市盈率-动态:PE
            17、市净率:PB
            18、总市值:TOTAL_MV
            19、流通市值:FLOAT_MV
            20、涨速:RISE
            21、5分钟涨跌:5MIN
            22、60日涨跌幅:60DAY
            23、年初至今涨跌幅:1YEAR
        新浪财经：
            1、代码:TS_CODE
            2、名称:NAME
            3、最新价:PRICE
            4、涨跌额:CHANGE
            5、涨跌幅:PCT_CHANGE
            6、买入:BUY
            7、卖出:SALE
            8、昨收:CLOSE
            9、今开:OPEN
            10、最高:HIGH
            11、最低:LOW
            12、成交量:VOLUME
            13、成交额:AMOUNT
            14、时间戳:TIME

    """
    if src == "dc":
        return get_stock_all_a_dc(page_count, proxies)
    elif src == "sina":
        return get_stock_all_a_sina(interval, page_count, proxies)
    else:
        return get_stock_all_a_dc(page_count, proxies)


def get_stock_all_a_dc(page_count: Optional[int] = None,
                       proxies: Optional[dict] = {}) -> pd.DataFrame:
    """
    东方财富网-沪深京 A 股-实时行情
    https://quote.eastmoney.com/center/gridlist.html#hs_a_board
    :return: 实时行情
    :rtype: pandas.DataFrame
        1、序号:RANK
        2、代码:TS_CODE
        3、名称:NAME
        4、最新价:PRICE
        5、涨跌幅:PCT_CHANGE
        6、涨跌额:CHANGE
        7、成交量:VOLUME
        8、成交额:AMOUNT
        9、振幅:SWING
        10、最高:HIGH
        11、最低:LOW
        12、今开:OPEN
        13、昨收:CLOSE
        14、量比:VOL_RATIO
        15、换手率:TURNOVER_RATE
        16、市盈率-动态:PE
        17、市净率:PB
        18、总市值:TOTAL_MV
        19、流通市值:FLOAT_MV
        20、涨速:RISE
        21、5分钟涨跌:5MIN
        22、60日涨跌幅:60DAY
        23、年初至今涨跌幅:1YEAR
    """
    url = "http://82.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "50000",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:0 t:6,m:0 t:80,m:1 t:2,m:1 t:23,m:0 t:81 s:2048",
        "fields": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,f23,f24,f25,f22,f11,f62,f128,f136,f115,f152",
        "_": "1623833739532",
    }
    if page_count:
        params["pz"] = 20
    r = requests.get(url, params=params, proxies=proxies)
    data_json = r.json()
    if not data_json["data"]["diff"]:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["data"]["diff"])
    temp_df.columns = [
        "_",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "成交量",
        "成交额",
        "振幅",
        "换手率",
        "市盈率-动态",
        "量比",
        "5分钟涨跌",
        "代码",
        "_",
        "名称",
        "最高",
        "最低",
        "今开",
        "昨收",
        "总市值",
        "流通市值",
        "涨速",
        "市净率",
        "60日涨跌幅",
        "年初至今涨跌幅",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df.reset_index(inplace=True)
    # temp_df["index"] = temp_df.index + 1
    # temp_df.rename(columns={"index": "序号"}, inplace=True)
    temp_df = temp_df[
        [
            # "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "涨跌额",
            "成交量",
            "成交额",
            "振幅",
            "最高",
            "最低",
            "今开",
            "昨收",
            "量比",
            "换手率",
            "市盈率-动态",
            "市净率",
            "总市值",
            "流通市值",
            "涨速",
            "5分钟涨跌",
            "60日涨跌幅",
            "年初至今涨跌幅",
        ]
    ]

    temp_df["代码"] = temp_df["代码"].apply(format_stock_code)
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    temp_df["昨收"] = pd.to_numeric(temp_df["昨收"], errors="coerce")
    temp_df["量比"] = pd.to_numeric(temp_df["量比"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    temp_df["市盈率-动态"] = pd.to_numeric(temp_df["市盈率-动态"], errors="coerce")
    temp_df["市净率"] = pd.to_numeric(temp_df["市净率"], errors="coerce")
    temp_df["总市值"] = pd.to_numeric(temp_df["总市值"], errors="coerce")
    temp_df["流通市值"] = pd.to_numeric(temp_df["流通市值"], errors="coerce")
    temp_df["涨速"] = pd.to_numeric(temp_df["涨速"], errors="coerce")
    temp_df["5分钟涨跌"] = pd.to_numeric(temp_df["5分钟涨跌"], errors="coerce")
    temp_df["60日涨跌幅"] = pd.to_numeric(temp_df["60日涨跌幅"], errors="coerce")
    temp_df["年初至今涨跌幅"] = pd.to_numeric(temp_df["年初至今涨跌幅"], errors="coerce")
    temp_df.columns = [
        # "RANK",
        "TS_CODE",
        "NAME",
        "PRICE",
        "PCT_CHANGE",
        "CHANGE",
        "VOLUME",
        "AMOUNT",
        "SWING",
        "HIGH",
        "LOW",
        "OPEN",
        "CLOSE",
        "VOL_RATIO",
        "TURNOVER_RATE",
        "PE",
        "PB",
        "TOTAL_MV",
        "FLOAT_MV",
        "RISE",
        "5MIN",
        "60DAY",
        "1YEAR",
    ]
    # 指定要转换为 float 类型的列
    cols_to_convert = ['PRICE', 'PCT_CHANGE', 'CHANGE', "VOLUME", "AMOUNT", "SWING",
                       'HIGH', "LOW", "OPEN", "CLOSE", "VOL_RATIO", "TURNOVER_RATE", "PE", "PB", "TOTAL_MV", "FLOAT_MV",
                       "RISE", "5MIN", "60DAY", "1YEAR"
                       ]
    # 使用 to_numeric() 方法将指定的列转换为 float 类型，并将非数值类型的数据转换为 NaN
    temp_df[cols_to_convert] = temp_df[cols_to_convert].apply(pd.to_numeric, errors='coerce')
    # 使用 fillna() 方法将 NaN 值替换为 0
    df_filled = temp_df.fillna(0)
    df_sorted = df_filled.sort_values(by='PCT_CHANGE', ascending=False).reset_index(drop=True)
    return df_sorted


def _get_zh_a_page_count() -> int:
    """
    所有股票的总页数
    https://vip.stock.finance.sina.com.cn/mkt/#hs_a
    :return: 需要采集的股票总页数
    :rtype: int
    """
    res = requests.get(zh_sina_a_stock_count_url)
    page_count = int(re.findall(re.compile(r"\d+"), res.text)[0]) / 80
    if isinstance(page_count, int):
        return page_count
    else:
        return int(page_count) + 1


def get_stock_all_a_sina(interval: Optional[int] = 3, page_count: Optional[int] = None,
                         proxies: Optional[dict] = {}) -> pd.DataFrame:
    """
    新浪财经-所有 A 股的实时行情数据; 重复运行本函数会被新浪暂时封 IP
    https://vip.stock.finance.sina.com.cn/mkt/#hs_a
    :param interval:请求间隔时间
    :param page_count:限制抓取页数
    :param proxies: 代理ip {}
    :return: 所有股票的实时行情数据
    :rtype: pandas.DataFrame
        1、代码:TS_CODE
        2、名称:NAME
        3、最新价:PRICE
        4、涨跌额:CHANGE
        5、涨跌幅:PCT_CHANGE
        6、买入:BUY
        7、卖出:SALE
        8、昨收:CLOSE
        9、今开:OPEN
        10、最高:HIGH
        11、最低:LOW
        12、成交量:VOLUME
        13、成交额:AMOUNT
        14、时间戳:TIME

    """
    big_df = pd.DataFrame()
    if not page_count:
        page_count = _get_zh_a_page_count()
    zh_sina_stock_payload_copy = zh_sina_a_stock_payload.copy()
    for page in tqdm(
            range(1, page_count + 1), leave=False, desc="Please wait for a moment"
    ):
        zh_sina_stock_payload_copy.update({"page": page})
        r = requests.get(
            zh_sina_a_stock_url, headers=zh_sina_a_stock_headers,
            cookies=zh_sina_a_stock_cookies,
            params=zh_sina_stock_payload_copy,
            proxies=proxies,
        )
        data_json = r.json()
        big_df = pd.concat(
            [big_df, pd.DataFrame(data_json)], ignore_index=True
        )

        time.sleep(interval)

    big_df = big_df.astype(
        {
            "trade": "float",
            "pricechange": "float",
            "changepercent": "float",
            "buy": "float",
            "sell": "float",
            "settlement": "float",
            "open": "float",
            "high": "float",
            "low": "float",
            "volume": "float",
            "amount": "float",
            "per": "float",
            "pb": "float",
            "mktcap": "float",
            "nmc": "float",
            "turnoverratio": "float",
        }
    )
    big_df.columns = [
        "代码",
        "_",
        "名称",
        "最新价",
        "涨跌额",
        "涨跌幅",
        "买入",
        "卖出",
        "昨收",
        "今开",
        "最高",
        "最低",
        "成交量",
        "成交额",
        "时间戳",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "买入",
            "卖出",
            "昨收",
            "今开",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "时间戳",
        ]
    ]
    big_df["代码"] = big_df["代码"].apply(format_stock_code)
    big_df.columns = [
        "TS_CODE",
        "NAME",
        "PRICE",
        "CHANGE",
        "PCT_CHANGE",
        "BUY",
        "SALE",
        "CLOSE",
        "OPEN",
        "HIGH",
        "LOW",
        "VOLUME",
        "AMOUNT",
        "TIME",

    ]
    df_sorted = big_df.sort_values(by='PCT_CHANGE', ascending=False).reset_index(drop=True)
    return df_sorted


def get_realtime_quotes_dc(symbols="688553"):
    """
    https://quote.eastmoney.com/sh601096.html
    """
    symbols = str(symbols).split(",")[0]
    url = "https://push2.eastmoney.com/api/qt/stock/get"
    symbol = re.search(r"(\d+)", symbols, re.S | re.M).group(1)
    # print(symbol)
    params = {
        "invt": "2",
        "fltt": "1",
        # "cb": "jQuery35108939078769986013_1701853424476",
        "fields": "f58,f734,f107,f57,f43,f59,f169,f301,f60,f170,f152,f177,f111,f46,f44,f45,f47,f260,f48,f261,f279,f277,f278,f288,f19,f17,f531,f15,f13,f11,f20,f18,f16,f14,f12,f39,f37,f35,f33,f31,f40,f38,f36,f34,f32,f211,f212,f213,f214,f215,f210,f209,f208,f207,f206,f161,f49,f171,f50,f86,f84,f85,f168,f108,f116,f167,f164,f162,f163,f92,f71,f117,f292,f51,f52,f191,f192,f262,f294,f295,f748,f747",
        "secid": f"0.{symbol}",
        "ut": "fa5fd1943c7b386f172d6893dbfba10b",
        "wbp2u": "|0|0|0|web",
        "_": _get_current_timestamp()
    }
    if not verify_stock_or_index(symbols):
        params["secid"] = f"1.{symbol}"
    # print(params["secid"])
    response = requests.get(url, headers=rtqv.dc_cookies, cookies=rtqv.dc_headers, params=params)
    data_info = response.json()["data"]
    if not data_info:
        return pd.DataFrame()
    name = data_info["f58"]
    open = data_info["f46"]  # / 100
    high = data_info["f44"]  # / 100
    pre_close = data_info["f60"]  # / 100
    low =  data_info["f45"] # / 100
    price = data_info["f43"]  # / 100 if data_info["f43"] != "-" else ""
    b5_v = format_str_to_float(data_info["f12"])
    b5_p = data_info["f11"]  # / 100 if data_info["f11"] != "-" else ""
    b4_v = format_str_to_float(data_info["f14"])
    b4_p = data_info["f13"]  # / 100 if data_info["f13"] != "-" else ""
    b3_v = format_str_to_float(data_info["f16"])
    b3_p = data_info["f15"]  # / 100 if data_info["f15"] != "-" else ""
    b2_v = format_str_to_float(data_info["f18"])
    b2_p = data_info["f17"]  # / 100 if data_info["f17"] != "-" else ""
    b1_v = format_str_to_float(data_info["f20"])
    b1_p = data_info["f19"]  # / 100 if data_info["f19"] != "-" else ""
    a5_v = format_str_to_float(data_info["f32"])
    a5_p = data_info["f31"]  # / 100 if data_info["f31"] != "-" else ""
    a4_v = format_str_to_float(data_info["f34"])
    a4_p = data_info["f33"]  # / 100 if data_info["f33"] != "-" else ""
    a3_v = format_str_to_float(data_info["f36"])
    a3_p = data_info["f35"]  # / 100 if data_info["f35"] != "-" else ""
    a2_v = format_str_to_float(data_info["f38"])
    a2_p = data_info["f37"]  # / 100 if data_info["f38"] != "-" else ""
    a1_v = format_str_to_float(data_info["f40"])
    a1_p = data_info["f39"]  # / 100 if data_info["f39"] != "-" else ""
    date_time = timestemp_to_time(data_info["f86"])
    date = date_time[0:10]
    times = date_time[10:]
    volume = format_str_to_float(data_info["f47"])
    amount = format_str_to_float(data_info["f48"])
    bid = format_str_to_float(data_info["f19"])
    ask = format_str_to_float(data_info["f39"])
    code = symbols
    data_list = [[name, open, pre_close, price, high, low, bid, ask, volume, amount,
                  b1_v, b1_p, b2_v, b2_p, b3_v, b3_p, b4_v, b4_p, b5_v, b5_p,
                  a1_v, a1_p, a2_v, a2_p, a3_v, a3_p, a4_v, a4_p, a5_v, a5_p, date, times, code]]
    df = pd.DataFrame(data_list, columns=rtqv.LIVE_DATA_COLS)
    df["DATE"] = df["DATE"].apply(format_date_str)
    df["ASK"] = df["ASK"].apply(format_dc_str)
    df["OPEN"] = df["OPEN"].apply(format_dc_str)
    df["HIGH"] = df["HIGH"].apply(format_dc_str)
    df["LOW"] = df["LOW"].apply(format_dc_str)
    df["PRE_CLOSE"] = df["PRE_CLOSE"].apply(format_dc_str)
    df["BID"] = df["BID"].apply(format_dc_str)
    df["A1_P"] = df["A1_P"].apply(format_dc_str)
    df["A2_P"] = df["A2_P"].apply(format_dc_str)
    df["A3_P"] = df["A3_P"].apply(format_dc_str)
    df["A4_P"] = df["A4_P"].apply(format_dc_str)
    df["A5_P"] = df["A5_P"].apply(format_dc_str)
    df["PRICE"] = df["PRICE"].apply(format_dc_str)
    df["B1_P"] = df["B1_P"].apply(format_dc_str)
    df["B2_P"] = df["B2_P"].apply(format_dc_str)
    df["B3_P"] = df["B3_P"].apply(format_dc_str)
    df["B4_P"] = df["B4_P"].apply(format_dc_str)
    df["B5_P"] = df["B5_P"].apply(format_dc_str)
    new_order = rtqv.LIVE_DATA_COLS_REINDEX
    df = df[new_order]
    return df


def format_dc_str(x):
    return float(x / 100) if x != "-" else 0


def format_str_to_float(x):
    try:
        return float(x) if x != "" else 0
    except:
        return 0


if __name__ == '__main__':
    # df = realtime_quote(ts_code="000688.SH,000010.SH,000012.SH,399005.SZ", src="sina")
    # df = realtime_list(src="sina", page_count=1)
    # print(df)
    ts_code = '399005.SZ'
    ts_code = '000001.SZ'
    # ts_code = '836149.BJ'
    # ts_code = '600000.SH'
    # ts_code = '000001.SH'
    # ts_code = '000010.SH'
    ts_code = '688111.SH'
    df = realtime_quote(src="dc", ts_code=ts_code)
    print(df)





#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
token 验证
Created on 2023/12/06
@author: Monday
@group : waditu
@contact:
"""
import datetime


def timestemp_to_time(time_stamp, form_date="%Y-%m-%d %H:%M:%S"):
    time_stamp = int(str(time_stamp)[0:10])
    date_array = datetime.datetime.fromtimestamp(time_stamp)
    other_style_time = date_array.strftime(form_date)
    return str(other_style_time)


def get_current_date(date_format='%Y-%m-%d %H:%M:%S', hours_num=0, minutes_num=0, is_time=False):
    """hours_num -1 #获取前一小时  +1  后一个小时"""
    if hours_num == 0 and minutes_num == 0:
        current_time = datetime.datetime.now().strftime(date_format)
    elif hours_num != 0 and minutes_num == 0:
        current_time = (datetime.datetime.now() + datetime.timedelta(hours=hours_num)).strftime(date_format)
    else:
        current_time = (datetime.datetime.now() + datetime.timedelta(minutes=minutes_num)).strftime(date_format)
    if not is_time:
        return current_time
    else:
        return datetime.datetime.strptime(current_time, date_format)



import json

import requests
try:
    import sseclient
except:
    raise ImportError("sseclient not found, please install it using 'pip install sseclient-py'.")


from tushare import get_token

BASE_URL = "http://api.waditu.com/dataapi"
# BASE_URL = "http://127.0.0.1:8000/dataapi"
API_KEY_PREFIX = "tsgpt-"


class LlmClent:
    def __init__(self, token=None, timetout=120):
        if not token:
            token = get_token()
        self.token = token
        self.timeout = timetout

    def chat_query(self, model, messages, temperature=None, max_tokens=None, pretty=False):
        """
        model string 模型名称， doubao-pro-128k
        messages list 消息列表
            [
                {
                  "role": "user",
                  "content": "Hello World"
                }
            ]
        pretty bool 是否只返回回答内容文本
        """
        resp = requests.post(
            f'{BASE_URL}/llm/{model}',
            json={"params": {
                "stream": False,
                "messages": messages,
                "temperature": temperature,
                "max_tokens": max_tokens
            }},
            headers={"Authorization": f"tstoken-{self.token}"},
            timeout=self.timeout
        )
        if resp.status_code != 200:
            raise Exception(f"请求出现错误，{resp.content}")

        resp_data = resp.json()
        if resp_data.get('code') not in (0, None):
            raise Exception(resp_data['msg'])
        if pretty:
            return resp_data['choices'][0]["message"]["content"]
        else:
            return resp_data

    def chat_stream(self, model, messages, temperature=None, max_tokens=None, pretty=False):
        """
        model string 模型名称， doubao-pro-128k
        messages list 消息列表
            [
                {
                  "role": "user",
                  "content": "Hello World"
                }
            ]
        pretty bool 是否只返回回答内容文本
        """
        resp = requests.post(
            f'{BASE_URL}/llm/{model}',
            json={"params": {
                "stream": True,
                "messages": messages,
                "temperature": temperature,
                "max_tokens": max_tokens
            }},
            headers={"Authorization": f"tstoken-{self.token}"},
            timeout=self.timeout, stream=True
        )
        if resp.status_code != 200:
            raise Exception(f"请求出现错误，{resp.content}")

        for e in sseclient.SSEClient(resp).events():
            e_data = json.loads(e.data)
            if pretty:
                yield e_data["choices"][0]["delta"]["content"]
            else:
                yield e_data


def test_chart_query():
    c = LlmClent()
    dd = c.chat_query("doubao-pro-128k", [{
        "role": "user",
        "content": "你好"
    }])
    print(dd)
    dd = c.chat_query("doubao-pro-128k", [{
        "role": "user",
        "content": "你好"
    }], pretty=True)
    print(dd)


def test_chart_stream():
    c = LlmClent()
    dd = c.chat_stream("doubao-pro-128k", [
        {
            "role": "user",
            "content": "你好"
        }
    ])
    for d in dd:
        print(d)

    dd = c.chat_stream("doubao-pro-128k", [
        {
            "role": "user",
            "content": "你好"
        }
    ], pretty=True)
    for d in dd:
        print(d)
